/********************************************************************
 *                   NASA CONFIDENTIAL INFORMATION                  *
 *               (c)1997-2001 Nasa Ames Research Center		    *
 *                        All Rights Reserved                       *
 *                                                                  *
 *  This program contains confidential and proprietary information  *
 *  of NASA Ames Research Center, any reproduction, disclosure,     *
 *  or use in whole or in part is expressly prohibited, except as   *
 *  may be specifically authorized by prior written agreement.      *
 *                                                                  *
 ********************************************************************/

package gov.nasa.arc.brahms.atm;

/*
ATM Model by Alessandro Acquisti acquisti@riacs.edu


COMPLETE VERSION


THE STORY:
This simulation models the simple life of laborious Berkeley students: studying, eating, and getting just enough cash to afford some staple. 
Students spend most of their time studying, but get hungrier as the time goes by. When they are particularly hungry (the level varying depending on the student), they decide to move to one of the two restaurants in town. Students choose the restaurant according to how much money they are carrying. If a student does not have enough money for the cheapest restaurant, she will decide to pass first by the ATM of the bank where she has her account.  

When she arrives at the ATM, the student inserts her bank card and attempt to remember the PIN associated to her account. The ATM gives students 3 attempts to digit the correct PIN, before refusing the card altogether. The ATM communicates with the central bank computer to verify the correctness of the information provided by the student. If the bank computer communicates to the ATM that the PIN is correct and that the student has enough balance in her account, the ATM will dispense the cash and
will print an invoice with the account number and the remaining balance.

Students need to have enough balance in their  accounts to take out
cash: if they attempt to take out more money than they have, the bank computer
will notify the students (through the ATM) of the remaining amount of dollars in the
account. The student will modify her habits accordingly, and take out just exactly the
remaining dollars. 


THE CAST (AGENTS and OBJECTS):
Students: Kim, Alex
Bank computers: Bank of America, Wells Fargo
Restaurants: Blakes, Raleighs
Studying Places: South Hall, Spraul Hall
Clock: the Campanile
ATMs: one ATM for each bank


NOTES:
- attributes and relations scopes are all set to public. private and protected scopes are admitted but not yet implemented in the current version of the language
- the simulation halts at the end of a Berkeley's student hard work day: 20 hours (sleep is not considered!)
- after taking money from the ATMs, student pass by the Hall before getting to the Restaurants
- objects are not given wait activities since detactables/facts do the job (see Tutorial text)


SUGGESTED CHANGES FOR FURTHER MODELING:
- start using private/public/protected scope (see above)
- change ATM and BANK to agents


FURTHER COMPLICATIONS:
- what if each bank has more than one ATM? can you make the agent choose the closest one?
- can  you model the objects (say, Banks and ATMs) as agents? what will change? do you notice changes in the behavior related to detectables and communications?
- what if more students go to the same ATM at the same time? can you make the ATM handle just one order at a time, but the Bank central computer multiple orders coming from several ATMs?
- can you make the agents randomly meet and interact (e.g., say hello each to the other)?
- what if you want the ATM to remember the number of the cards if has refused?


ABOUT THIS FILE:
This file contains the description of the ATM machine. The ATM communicates with the Student and the central bank computer
(called 'Bank'), dispenses cash (or refuses to do so), and  prints invoices


VERSION HISTORY
//VERSION 0.1, February 2001
//VERSION 0.9, April 2001
//VERSION 0.91 April 30, 2001
//VERSION 0.99 August 8, 2001
//VERSION 0.999 December 8, 2001

*/




class Atm {

	display: "Atm";
  	cost: 0.0;
 	resource: true;

	attributes:
		public int currentAccountCode;
		public int currentAccountPin;
		public boolean sessionIsOn;
		public boolean checkedAccountCode;
		public boolean checkedAccountPin;
		public boolean pinChecked;
		public boolean askedVerification;
		public boolean cashCanBeDispensed;
		public int errorCount;
		public boolean pinIsWrong;
		public boolean pinAsked;
		public boolean waitReply;
		public boolean amountAsked;
		public boolean amountChecked;
		public double currentAmountRequested;
		public boolean cashDispensed;
		public double cashToDispense;
		public boolean RepliedVerification;


	relations:
		public Bank ownedbyBank;

		//ATTRIBUTES AND RELATIONS SCOPES ARE ALL SET TO PUBLIC. PRIVATE AND PROTECTED SCOPES ARE ADMITTED BUT NOT YET IMPLEMENTED IN THE CURRENT VERSION OF THE LANGUAGE

	initial_beliefs:
		(current.pinIsWrong != false);
		(current.waitReply = false);
		(current.amountAsked = false);
		(current.RepliedVerification = false);
		(current.amountChecked = false);
		(current.currentAmountRequested = 0.0);
		(current.askedVerification = false);
		(current.cashDispensed != false);
		(current.sessionIsOn = false);
		(current.cashToDispense = 0.0);

	initial_facts:
		(current.sessionIsOn = false);
		(current.checkedAccountCode = false);
		(current.checkedAccountPin = false);
		(current.pinChecked = false);
		(current.askedVerification = false);
		(current.cashCanBeDispensed != false);
		(current.currentAccountPin = 0);
		(current.currentAccountCode = 0);
		(current.errorCount = 0);
		(current.pinAsked = false);
		(current.waitReply = false);
		(current.RepliedVerification = false);
		(current.amountAsked = false);
		(current.amountChecked = false);
		(current.currentAmountRequested = 0.0);
		(current.cashDispensed != false);
		(current.pinIsWrong != false);
		(current.cashToDispense = 0.0);

	activities:

		primitive_activity getAccount() {
					max_duration: 4;
				}

		primitive_activity getPin() {
					max_duration: 5;
				}

		primitive_activity getAmount() {
					max_duration: 5;
				}

		primitive_activity comparePins() {
					max_duration: 4;
				}

		primitive_activity matchPin_No() {
					max_duration: 3;
				}


		primitive_activity matchPin_Yes() {
					max_duration: 3;
				}

		primitive_activity returnCardandMoney() {
					max_duration: 9;
				}

		primitive_activity returnCardwithNoMoney() {
					max_duration: 6;
				}

		primitive_activity waitStudentReply() {
					max_duration: 3000;
		}
	
		primitive_activity waitBankReply() {
					max_duration: 3000;
		}

		primitive_activity processPinReply() {
					max_duration: 4;
		}

		primitive_activity processAskPin() {
					max_duration: 3;
		}


		primitive_activity processAskBankVerification() {
					max_duration: 4;
		}

		primitive_activity processAskAmount() {
					max_duration: 2;
		}
		
		communicate askPin(Student std) {
					max_duration: 1;
					with: std;
					about:
						send(current.pinAsked = true);
					
					when: end;
		}


		communicate askAmount(Student std) {
					max_duration: 1;
					with: std;
					about:
						send(current.amountAsked = true);
					
					when: end;
		}

		//NOTE THAT ATMs AND BANKs DO NOT NEED WAIT ACTIVITIES LIKE THE STUDENTS; THIS ARE DUE TO THE FACT THAT THEY ARE OBJECTS AND THE WAY OBJECTS DETECT FACTS - SEE THE TUTORIAL TEXT FOR FURTHER DETAILS

		communicate askPinAgain(Student std) {

					max_duration: 1;
					with: std;
					about:
		
					//	send(current.pinIsWrong = current.pinIsWrong),
						send(current.pinIsWrong = unknown),
						send(std.pinRemembered = false),
						send(std.pinCommunicated = false),
						send(current.pinAsked = true),
						send(current.amountAsked = false),
						send(std.waitAtmRepliesVerification = true),
						send(std.repeatPin = true);

					when: end;
		}


		communicate replyStudent(Student std, BankCard bkc) {

					max_duration: 1;
					with: std;
					about:
		
						send(current.cashDispensed = current.cashDispensed),
						send(std contains bkc),
						send(current.cashToDispense = current.cashToDispense);
					
					
					when: end;
		}
		
		communicate askBankVerification_1(Bank ba3) {
					max_duration: 1;
					with: ba3;
					about:
					//	send(current.currentAccountCode = current.currentAccountCode),
					//	send(current.currentAccountPin = current.currentAccountPin),
					//	send(current.currentAmountRequested = current.currentAmountRequested),
						send(current.currentAccountCode = unknown),
						send(current.currentAccountPin = unknown),
						send(current.currentAmountRequested = unknown),
						send(current.askedVerification = true);
					
					when: end;
		}


		//THIS IS USED WHEN THE STUDENTS TRIES TO GET OUR MORE CASH THAN SHE HAS
		communicate adjustBalance(Account bka, Student std) {
					max_duration: 1;
					with: std;
					about:

						send(std.preferredCashOut = bka.balance);
					
					when: end;
		}

		//THIS IS USED WHEN THE STUDENTS TRIES TO GET OUR EACTLY THE REMAINING CASH
		communicate lastBalance(Account bka, Student std) {
					max_duration: 1;
					with: std;
					about:
						send(std.preferredCashOut = 0.0),
						send(std.needCash = false);
					
					when: end;
		}


		create_object createInvoice(Invoice atm_invoice, Building bd) {
					max_duration: 4;
					action:new;
					source:Invoice;

					destination: atm_invoice;
					location: bd;
					when: start;
		}
		


	workframes:

		
		workframe wf_getAccount {

					repeat: true;

					variables:
						forone(BankCard) bkc; 

					when(knownval(current contains bkc) and
						knownval(current.sessionIsOn = false) and
						knownval(current.checkedAccountCode = false))
						
						
					do {
						getAccount();
						conclude((current.currentAccountCode = bkc.code ), bc:100, fc:100);
						conclude((current.sessionIsOn = true), bc:100, fc:100);
						conclude((current.checkedAccountCode = true), bc:100, fc:100);
						
					}
		}

		workframe wf_askPin {

					repeat: true;

					variables:
						forone(BankCard) bkc; 
						forone(Student) std;

			
					when(knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.checkedAccountPin = false) and
						knownval(current.checkedAccountCode = true) and
						knownval(current.pinAsked = false))
						
						
					do {
						processAskPin();
						askPin(std);
						conclude((current.pinAsked = true), bc:100, fc:100);
						
						
					}
		}
		
	


		workframe wf_getPin {

					repeat: true;

					variables:
						forone(BankCard) bkc; 
						forone(Student) std;
					
					when(knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.checkedAccountPin = false) and
						knownval(current.checkedAccountCode = true) and
						knownval(std.pinCommunicated = true))
						
					do {
						getPin();
						conclude((current.currentAccountPin = std.believedPin), bc:100, fc:100);
						conclude((current.checkedAccountPin = true), bc:100, fc:100);
						
					}
		}
		

		workframe wf_askAmountWanted {

					repeat: true;

					variables:
						forone(BankCard) bkc; 
						forone(Student) std;

			
					when(knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.checkedAccountPin = true) and
						knownval(current.checkedAccountCode = true) and
						knownval(current.pinAsked = true) and
						knownval(current.amountAsked = false))
						
						
					do {
						processAskAmount();
						askAmount(std);	
						conclude((current.amountAsked = true), bc:100, fc:100);
	
					}
		}


		
		workframe wf_getAmount {

					repeat: true;

					variables:
						forone(BankCard) bkc; 
						forone(Student) std;
					
					when(knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.checkedAccountPin = true) and
						knownval(current.checkedAccountCode = true) and
						knownval(std.pinCommunicated = true) and
						knownval(current.amountAsked = true) and
						knownval(std.amountCommunicated = true) and
						knownval(current.amountChecked = false))
						
						
					do {
						getAmount();
						conclude((current.currentAmountRequested = std.preferredCashOut), bc:100, fc:100);
						conclude((current.amountChecked = true), bc:100, fc:100);
						
					}
		}





		workframe wf_askBankVerification {

					
					repeat: true;

					priority: 10;
					// NECESSARY, OTHERWISE THE SECOND TIME AROUND, IF THE PIN WAS WRONG, THE ATM DOES NOT TALK WITH THE BANK


					variables:
						forone(Account) bka;   
						forone(BankCard) bkc; 
						forone(Bank) ba3;
					
					when(knownval(current contains bkc) and
						knownval(current.checkedAccountPin = true) and
						knownval(current.checkedAccountCode = true) and
						knownval(current ownedbyBank ba3) and
						knownval(current.askedVerification = false) and
						knownval(current.amountChecked = true))						
						
						
					do {
						processAskBankVerification();
						askBankVerification_1(ba3);
						conclude((current.askedVerification = true), bc:100, fc:100);
						conclude((current.pinChecked =  false), bc:100, fc:100);
						//THERE IS AN ISSUE RELATED TO TIMING HERE: IF OBJECTS REACT ON FACTS,
						// THE BANK WILL KNOW OF THIS IMMEDIATELY AS SOONS AS FACTS ARE CREATED,
						//WITHOUT ANY NEED FOR COMMUNICATIONS. WE MIGHT MODEL ATM AND BANK AS AGENT AND MAKE THEM ACT ON BELIEFS
						//OR USE A NEW CONSTRUCT IN THE LANGUAGE, CALLED 'DATAFRAMES' - SEE ONLINE LANGUAGE SPECIFICATIONS

						
					}
		}



		workframe wf_matchPin_Yes {

					repeat: true;

					variables:
						forone(Bank) ba3;
						forone(BankCard) bkc;
						forone(Student) std;
						forone(Account) bka;
				
											
					when(knownval(current ownedbyBank ba3) and
						knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.currentAccountCode = bka.code) and
						knownval(ba3.correctPin = true) and
						knownval(current.pinChecked = false) and
						knownval(ba3.enoughBalance = true) and
						knownval(ba3.hasReplied = true) and
						knownval(bka.balance != 0.0))
						
						
					do {

						conclude((current.cashCanBeDispensed = true), bc:100, fc:100);
						conclude((current.cashToDispense = std.preferredCashOut), bc:100, fc:100);
						conclude((current.pinIsWrong = false), bc:100, fc:100);
						conclude((current.pinChecked = true), bc:100, fc:100);
						conclude((current.errorCount = 0), bc:100, fc:100);
						matchPin_Yes();										
					}
		}

		//THIS IS USED WHEN THE STUDENTS TRIES TO GET OUR MORE CASH THAN SHE HAS
		workframe wf_adjustBalance {

					repeat: true;

					variables:
						forone(Bank) ba3;
						forone(BankCard) bkc;
						forone(Student) std;
						forone(Account) bka;
				
											
					when(knownval(current ownedbyBank ba3) and
						knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.currentAccountCode = bka.code) and
						knownval(ba3.correctPin = true) and
						knownval(current.pinChecked = false) and
						knownval(ba3.enoughBalance = false) and
						knownval(ba3.hasReplied = true))

						
						
					do {
						conclude((current.cashCanBeDispensed = false), bc:100, fc:100);
						conclude((current.cashToDispense = 0.0), bc:100, fc:100);
						conclude((current.pinIsWrong = false), bc:100, fc:100);
						conclude((current.pinChecked = true), bc:100, fc:100);
						conclude((current.errorCount = 0), bc:100, fc:100);
						conclude((std.preferredCashOut = bka.balance), bc:100, fc:100); 
						//NOTE - HERE AND ELSEWHERE WE LET THE ATM CONCLUDE FACTS RELATED TO EITHER THE BANK OR THE STUDENT.
						//WHILE SOME KIND OF RATIONALE MIGHT BE OFFERED IN THIS SPECIFIC SCENARIO (E.G., THE ATM IS CONCLUDING FACTS RELATED TO THE BANK WHEN IT ACTUALLY MODIFIES SOME VALUES STORED IN THE BANK COMPUTER; 
						//IT CONCLUDES FACTS ABOUT THE STUDENTS SUCH AS THE FACT THAT THE STUDENT CONTAINS OR DOES NOT CONTAIN THE BANKCARD, ETC.), SUCH PRACTICE IS NOT ALWAYS METHODOLOGICALLY CORRECT AND SHOULD BE USED WITH CARE
						//IN THIS CASE IT MAKES THE CODE EASIER, AS OBJECTS (LIKE THE ATM AND THE BANK) ONLY OPERATE ON FACTS RATHER THAN BELIEFS. 
						//HOWEVER, THE USE OF A NEW LANGUAGE CONSTRUCT CALLED 'DATAFRAMES' WILL MAKE THIS KIND OF SOLUTION NO LONGER NECESSARY
						//BECAUSE INFORMATION PROCESSING OBJECTS WILL BE ABLE TO ACT ON BELIEFS THROUGH SUCH DATAFRAMES.
						//THE NEXT VERSION OF THIS TUTORIAL WILL INCLUDE DATAFRAMES AND REMOVE THE CODE WHERE THE ATM CONCLUDES FACTS ABOUT THE BANK OR THE STUDENT
						//SEE THE ONLINE LANGUAGE SPECIFICATIONS (WWW.AGENTISOLUTIONS.COM) FOR MORE INFORMATION ABOUT DATAFRAMES.

						adjustBalance(bka,std);										
					}
		}

		//THIS IS USED WHEN THE STUDENTS TRIES TO GET OUR EACTLY THE REMAINING CASH
		workframe wf_lastBalance {

					repeat: true;

					variables:
						forone(Bank) ba3;
						forone(BankCard) bkc;
						forone(Student) std;
						forone(Account) bka;
				
											
					when(knownval(current ownedbyBank ba3) and
						knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(current.currentAccountCode = bka.code) and
						knownval(ba3.correctPin = true) and
						knownval(current.pinChecked = false) and
						knownval(ba3.enoughBalance = true) and
						knownval(ba3.hasReplied = true) and
						knownval(bka.balance = 0.0))
						
						
					do {
						conclude((current.cashCanBeDispensed = true), bc:100, fc:100);
						conclude((current.cashToDispense = std.preferredCashOut), bc:100, fc:100);
						conclude((current.pinIsWrong = false), bc:100, fc:100);
						conclude((current.pinChecked = true), bc:100, fc:100);
						conclude((current.errorCount = 0), bc:100, fc:100);
						conclude((std.preferredCashOut = 0.0), bc:100, fc:100);
						conclude((std.needCash = false), bc:100, fc:100);
						lastBalance(bka,std);										
					}
		}


		//THIS IS USED THE FIRST TIME A STUDENT GETS THE PIN WRONG. SHE WILL BE GIVEN TWO MORE ATTEMPTS
		workframe wf_matchPin_No {

					repeat: true;

					variables:
						forone(Bank) ba3;
						forone(BankCard) bkc;
						forone(Student) std;
		
											
					when(knownval(current ownedbyBank ba3) and
						knownval(current contains bkc) and
						knownval(std hasBankCard bkc) and
						knownval(ba3.correctPin = false) and
						knownval(current.errorCount < 2) and 
						knownval(current.pinAsked = true) and
						knownval(current.pinChecked = false)) 

						
						
					do {
						processAskPin();
						conclude((current.pinIsWrong = true), bc:100, fc:100);
						conclude((std.pinCommunicated = false), bc:100, fc:0);
						conclude((std.pinRemembered = false), bc:100, fc:0);
						conclude((std.repeatPin = true), bc:100, fc:0);
						// WITHOUT THESE TWO LINES ABOVE, THE BELIEF CANNOT BE COMMUNICATED TO THE STUDENT IN THE ASKPINAGAIN ACTIVITY.

						conclude((std.pinCommunicated = false), bc:100, fc:0);
						conclude((std.waitAtmRepliesVerification = true), bc:100, fc:0);
						//ONLY THE BELIEF. SO THAT WE CAN PASS TO AGENT THE BELIEF THAT HE DOES NOT NEED TO GIVE AMOUNT AGAIN
						askPinAgain(std);

						conclude((current.pinChecked != false), bc:100, fc:100);
						conclude((current.errorCount =  current.errorCount + 1), bc:100, fc:100);
						conclude((current.checkedAccountPin = false), bc:100, fc:100);
						conclude((current.askedVerification = false), bc:100, fc:100);
						//NOTE THE ASYMMETRIC POSITION; WE NEED IT HERE TO RESTART, BUT ELSEWHERE TO GO ON
						conclude((ba3.hasReplied = false), bc:100, fc:100);
						conclude((current.amountChecked = true), bc:100, fc:100);
						conclude((current.checkedAccountCode = true), bc:100, fc:100);
						conclude((current.amountAsked = true), bc:100, fc:100);
						conclude((current.pinAsked = true), bc:100, fc:100);
						conclude((ba3.hasReplied = false), bc:100, fc:100);
						conclude((ba3.correctPin != false), bc:100, fc:100);
						conclude((current.RepliedVerification = true), bc:100, fc:100);
					}
		}


		//THIS IS USED THE THIRD TIME A STUDENT GETS THE PIN WRONG. SHE WILL BE GIVEN THE CARD BACK
		workframe wf_matchPin_No_ThirdTime {

					repeat: true;

					variables:
						forone(Bank) ba3;
						forone(BankCard) bkc;
				
											
					when(knownval(current ownedbyBank ba3) and
						knownval(current contains bkc) and
						knownval(ba3.correctPin = false) and
						knownval(current.errorCount = 2) and
						knownval(current.pinChecked = false) and
						knownval(ba3.hasReplied = true))
						
						
					do {
						matchPin_No();
						conclude((current.cashCanBeDispensed = false), bc:100, fc:100);
						conclude((current.pinChecked = true), bc:100, fc:100);
						conclude((current.errorCount = 0), bc:100, fc:100);	
						conclude((current.pinIsWrong = true), bc:100, fc:100);
											
					}
		}
		
		workframe wf_returnCardandMoney {


					repeat: true;

					variables:
						forone(BankCard) bkc; 
						forone(Student) std;
						forone(Bank) ba3;
						forone(Account) bka;
						forone(Building) bd;
						forone(Invoice) atm_invoice;
						
					when(knownval(current contains bkc) and
						knownval(current.sessionIsOn = true) and
						knownval(current ownedbyBank ba3) and
						knownval(current.checkedAccountPin = true) and
						knownval(std hasBankCard bkc) and
						knownval(current.checkedAccountCode = true) and
						knownval(current.currentAccountCode = bka.code) and
						knownval(current.pinChecked = true) and
						knownval(current.cashCanBeDispensed = true) and
						knownval(current.location = bd))
			
	
					do {
						createInvoice(atm_invoice, bd);
						processPinReply();

						conclude((atm_invoice.code = bka.code), bc:100, fc:100);
						conclude((atm_invoice.balance = bka.balance), bc:100, fc:100);
						conclude((std contains atm_invoice), bc:100, fc:100);
						conclude((current.cashDispensed = true), bc:100, fc:100);
						conclude((current contains bkc is false), bc:100, fc:100);
						conclude((std contains bkc), bc:100, fc:100);
						replyStudent(std, bkc);
						conclude((current.sessionIsOn = false ), bc:100, fc:100);
						conclude((current.checkedAccountCode = false ), bc:100, fc:100);
						conclude((current.checkedAccountPin = false ), bc:100, fc:100);
						conclude((current.pinChecked = false), bc:100, fc:100);
						conclude((current.pinAsked = false), bc:100, fc:100);
						conclude((current.amountAsked = false), bc:100, fc:100);
						conclude((current.amountChecked = false), bc:100, fc:100);
						conclude((current.currentAccountPin = 0), bc:100, fc:100);
						conclude((current.currentAccountCode = 0), bc:100, fc:100);
						conclude((current.askedVerification = false), bc:100, fc:100);
						conclude((ba3.hasReplied = false), bc:100, fc:100);
						conclude((ba3.enoughBalance != false), bc:100, fc:100);
						conclude((ba3.correctPin != false), bc:100, fc:100);
						conclude((std.pinCommunicated = false), bc:100, fc:100);
						conclude((std.repeatPin = false), bc:100, fc:100);
						conclude((current.pinIsWrong != false), bc:100, fc:100);
						conclude((current.RepliedVerification = true), bc:100, fc:100);
					}
		}
		
				
		workframe wf_returnCardwithNoMoney {



					repeat: true;

					variables:
						forone(BankCard) bkc; 
						forone(Student) std;
						forone(Bank) ba3;
					
					when(knownval(current contains bkc) and
						knownval(current.sessionIsOn = true) and
						knownval(current ownedbyBank ba3) and
						knownval(current.checkedAccountPin = true) and
						knownval(std hasBankCard bkc) and
						knownval(current.checkedAccountCode = true) and
						knownval(current.pinChecked = true) and
						knownval(current.cashCanBeDispensed = false))
					
	
					do {
						processPinReply();

						conclude((current.cashDispensed = false), bc:100, fc:100);
						conclude((current.sessionIsOn = false), bc:100, fc:100);
						conclude((current contains bkc is false), bc:100, fc:100);
						conclude((std contains bkc), bc:100, fc:100);
					
						replyStudent(std, bkc);
						//WE NEED TO TELL THE STUDENT THAT NOW HE HAS THE CARD!!!!

						conclude((current.checkedAccountCode = false ), bc:100, fc:100);
						conclude((current.checkedAccountPin = false ), bc:100, fc:100);
						conclude((current.pinChecked = false), bc:100, fc:100);
						conclude((current.pinAsked = false), bc:100, fc:100);
						conclude((current.amountAsked = false), bc:100, fc:100);
						conclude((current.amountChecked = false), bc:100, fc:100);
						conclude((current.currentAccountPin = 0), bc:100, fc:100);
						conclude((current.currentAccountCode = 0), bc:100, fc:100);
						conclude((current.askedVerification = false), bc:100, fc:100);
						conclude((std.pinCommunicated = false), bc:100, fc:100);
						conclude((std.repeatPin = false), bc:100, fc:100);
						conclude((ba3.hasReplied = false), bc:100, fc:100);
						conclude((ba3.enoughBalance != false), bc:100, fc:100);
						conclude((ba3.correctPin != false), bc:100, fc:100);
						conclude((current.pinIsWrong != false), bc:100, fc:100);
						conclude((current.RepliedVerification = true), bc:100, fc:100);
					}
		}

	
}
