/********************************************************************
 *                   NASA CONFIDENTIAL INFORMATION                  *
 *               (c)1997-2001 Nasa Ames Research Center		    *
 *                        All Rights Reserved                       *
 *                                                                  *
 *  This program contains confidential and proprietary information  *
 *  of NASA Ames Research Center, any reproduction, disclosure,     *
 *  or use in whole or in part is expressly prohibited, except as   *
 *  may be specifically authorized by prior written agreement.      *
 *                                                                  *
 ********************************************************************/

package gov.nasa.arc.brahms.atm;

/*
ATM Model by Alessandro Acquisti acquisti@riacs.edu


COMPLETE VERSION


THE STORY:
This simulation models the simple life of laborious Berkeley students: studying, eating, and getting just enough cash to afford some staple. 
Students spend most of their time studying, but get hungrier as the time goes by. When they are particularly hungry (the level varying depending on the student), they decide to move to one of the two restaurants in town. Students choose the restaurant according to how much money they are carrying. If a student does not have enough money for the cheapest restaurant, she will decide to pass first by the ATM of the bank where she has her account.  

When she arrives at the ATM, the student inserts her bank card and attempt to remember the PIN associated to her account. The ATM gives students 3 attempts to digit the correct PIN, before refusing the card altogether. The ATM communicates with the central bank computer to verify the correctness of the information provided by the student. If the bank computer communicates to the ATM that the PIN is correct and that the student has enough balance in her account, the ATM will dispense the cash and
will print an invoice with the account number and the remaining balance.

Students need to have enough balance in their  accounts to take out
cash: if they attempt to take out more money than they have, the bank computer
will notify the students (through the ATM) of the remaining amount of dollars in the
account. The student will modify her habits accordingly, and take out just exactly the
remaining dollars. 


THE CAST (AGENTS and OBJECTS):
Students: Kim, Alex
Bank computers: Bank of America, Wells Fargo
Restaurants: Blakes, Raleighs
Studying Places: South Hall, Spraul Hall
Clock: the Campanile
ATMs: one ATM for each bank


NOTES:
- attributes and relations scopes are all set to public. private and protected scopes are admitted but not yet implemented in the current version of the language
- the simulation halts at the end of a Berkeley's student hard work day: 20 hours (sleep is not considered!)
- after taking money from the ATMs, student pass by the Hall before getting to the Restaurants
- objects are not given wait activities since detactables/facts do the job (see Tutorial text)


SUGGESTED CHANGES FOR FURTHER MODELING:
- start using private/public/protected scope (see above)
- change ATM and BANK to agents


FURTHER COMPLICATIONS:
- what if each bank has more than one ATM? can you make the agent choose the closest one?
- can  you model the objects (say, Banks and ATMs) as agents? what will change? do you notice changes in the behavior related to detectables and communications?
- what if more students go to the same ATM at the same time? can you make the ATM handle just one order at a time, but the Bank central computer multiple orders coming from several ATMs?
- can you make the agents randomly meet and interact (e.g., say hello each to the other)?
- what if you want the ATM to remember the number of the cards if has refused?


ABOUT THIS FILE:
This file contains the description of the bank's central computer. The Bank computer stores information related to owners of bank accounts.
The bank computer communicates with the ATM and gives it instructions about how to handle a transaction.


VERSION HISTORY
//VERSION 0.1, February 2001
//VERSION 0.9, April 2001
//VERSION 0.91 April 30, 2001
//VERSION 0.99 August 8, 2001
//VERSION 0.999 December 8, 2001

*/





class Bank {

	display: "Bank";
  	cost: 0.0;
 	resource: true;

	attributes:
		public string name;
		public int receivedAccountPin;
		public int receivedAccountCode;
		public boolean correctPin;
		public boolean readyToTransmit;
		public boolean receivedData;
		public boolean sessionIsOn;
		public boolean hasReplied;
		public boolean waitForInfo;
		public boolean balanceCalculated;
		public boolean balanceChecked;
		public boolean enoughBalance;
		public double newBalance;
		public double originalBalance;
		

	relations:
		
		//ATTRIBUTES AND RELATIONS SCOPES ARE ALL SET TO PUBLIC. PRIVATE AND PROTECTED SCOPES ARE ADMITTED BUT NOT YET IMPLEMENTED IN THE CURRENT VERSION OF THE LANGUAGE

	initial_facts:
		(current.receivedAccountPin != 0);
		(current.receivedAccountCode != 0);
		(current.correctPin != false);
		(current.readyToTransmit = false);
		(current.receivedData = false);
		(current.hasReplied = false);
		(current.waitForInfo = true);
		(current.balanceChecked = false);
		(current.balanceCalculated = false);
		(current.enoughBalance != false);

	activities:

		primitive_activity registerInfo() {
					max_duration: 5;
		}

		primitive_activity compareInfo_bad() {
					max_duration: 8;
		}

		primitive_activity compareInfo_ok() {
					max_duration: 8;
		}

		primitive_activity processReplyAtm() {
					max_duration: 2;
		}

		primitive_activity returnCardandMoney() {
					max_duration: 9;
		}

		primitive_activity returnCardwithNoMoney() {
					max_duration: 9;
		}

		primitive_activity waitInfo() {
					max_duration: 1000;
		}

		primitive_activity calculateBalance() {
					max_duration: 4;
		}

		primitive_activity checkBalance() {
					max_duration: 4;
		}


		communicate replyAtm(Atm at3, Account bka) {
					max_duration: 1;
					with: at3;
					about:

						send(current.correctPin = unknown),
						send(current.enoughBalance = unknown),
						send(bka.balance = unknown);


						//send(current.correctPin =  current.correctPin),
						//send(current.enoughBalance = current.enoughBalance),
						//send(bka.balance = bka.balance);
		
					when: end;
		}



	workframes:




		workframe wf_registerInfo {

					repeat: true;

					variables:
 
						forone(BankCard) bkc; 
						forone(Atm) at3;

					
					when(knownval(at3 ownedbyBank current) and
						known(at3.currentAccountCode) and
						known(at3.currentAccountPin) and
						knownval(at3.askedVerification = true) and
						knownval(current.receivedData = false) and
						knownval(current.readyToTransmit = false) and
						knownval(current.hasReplied = false))
						
					do {
						registerInfo();
						conclude((current.receivedAccountCode = at3.currentAccountCode), bc:100, fc:100);
						conclude((current.receivedAccountPin = at3.currentAccountPin), bc:100, fc:100);
						conclude((current.receivedData = true), bc:100, fc:100);
						}
		}


		//THE BALANCE IS FIRST CALCULATED. THEN, IF IT IS NEGATIVE, THE OPERATION IS ABORTED AND THE ORIGINAL CONDITION
		//OF THE ACCOUNT IS RESTORED (NEXT WORKFRAMES)

		workframe wf_calculateBalance {

					repeat: true;

					variables:
						foreach(Account) bka; 
						forone(Atm) at3;
					
					when(knownval(bka openedWithBank current) and
						known(at3.currentAccountCode) and
						known(at3.currentAccountPin) and
						knownval(at3.askedVerification = true) and
						known(current.receivedAccountCode) and
						known(current.receivedAccountPin) and
						knownval(current.receivedAccountCode = bka.code) and
						knownval(current.readyToTransmit = false) and
						knownval(current.hasReplied = false) and
						knownval(current.receivedData = true) and
						knownval(current.balanceCalculated = false))						
						
					do {
						conclude((current.originalBalance = bka.balance), bc:100, fc:100);
						calculateBalance();
						conclude((bka.balance = bka.balance - at3.currentAmountRequested), bc:100, fc:100);
						conclude((current.balanceCalculated = true), bc:100, fc:100);
						
					}
		}


		//THIS WORKFRAME IS TRIGGERED IF THE BANK FINDS ENOUGH BALANCE IN THE ACCOUNT
		workframe wf_checkBalance_ok {

					repeat: true;

					variables:
						foreach(Account) bka; 
						forone(Atm) at3;
					
					when(knownval(bka openedWithBank current) and
						known(at3.currentAccountCode) and
						known(at3.currentAccountPin) and
						knownval(at3.askedVerification = true) and
						known(current.receivedAccountCode) and
						known(current.receivedAccountPin) and
						knownval(current.receivedAccountCode = bka.code) and
						knownval(current.readyToTransmit = false) and
						knownval(current.hasReplied = false) and
						knownval(current.receivedData = true) and
						knownval(current.balanceCalculated = true) and
						knownval(current.balanceChecked = false) and
						knownval(bka.balance > -1.00)) 
						
						
					do {
						checkBalance();
						conclude((current.enoughBalance = true), bc:100, fc:100);
						conclude((current.balanceChecked = true), bc:100, fc:100);
						
					}
		}
		//NOTE THAT ATMs AND BANKs DO NOT NEED WAIT ACTIVITIES LIKE THE STUDENTS; THIS ARE DUE TO THE FACT THAT THEY ARE OBJECTS AND THE WAY OBJECTS DETECT FACTS - SEE THE TUTORIAL TEXT FOR FURTHER DETAILS



		//THIS WORKFRAME IS TRIGGERED IF THE BANK DOES NOT FIND ENOUGH BALANCE IN THE ACCOUNT
		workframe wf_checkBalance_bad {

					repeat: true;

					variables:
						foreach(Account) bka; 
						forone(Atm) at3;
					
					when(knownval(bka openedWithBank current) and
						known(at3.currentAccountCode) and
						known(at3.currentAccountPin) and
						knownval(at3.askedVerification = true) and
						known(current.receivedAccountCode) and
						known(current.receivedAccountPin) and
						knownval(current.receivedAccountCode = bka.code) and
						knownval(current.readyToTransmit = false) and
						knownval(current.hasReplied = false) and
						knownval(current.receivedData = true) and
						knownval(current.balanceCalculated = true) and
						knownval(current.balanceChecked = false) and
						knownval(bka.balance < 0.00)) 
						
					do {
						checkBalance();
						conclude((current.enoughBalance = false), bc:100, fc:100);
						conclude((bka.balance = bka.balance + at3.currentAmountRequested), bc:100, fc:100);
						conclude((current.balanceChecked = true), bc:100, fc:100);

					}
		}


		//THE BANK ALSO CHECK WHETHER THE PIN IS CORRECT. ONLY IF THE PIN IS CORRECT IT WILL PROCEED WITH THE OPERATIONS AND INFORM
		//THE ATM OF WHETHER THERE IS ENOUGH BALANCE IN THE ACCOUNT, OR THERE IS NOT
		workframe wf_compareInfo_ok {

					repeat: true;

					variables:
						foreach(Account) bka;   
					
					when(knownval(bka openedWithBank current) and
						known(current.receivedAccountCode) and
						known(current.receivedAccountPin) and
						knownval(current.receivedAccountCode = bka.code) and
						knownval(current.readyToTransmit = false) and
						knownval(current.hasReplied = false) and
						knownval(current.receivedData = true) and
						knownval(current.balanceChecked = true) and
						knownval(current.receivedAccountPin = bka.pin))
						
						
					do {
						compareInfo_ok();
						conclude((current.correctPin = true), bc:100, fc:100);
						conclude((current.readyToTransmit = true), bc:100, fc:100);
						
					}
		}


		workframe wf_compareInfo_bad {

					repeat: true;

					variables:
						foreach(Account) bka;   
					
					when(knownval(bka openedWithBank current) and
						known(current.receivedAccountCode) and
						known(current.receivedAccountPin) and
						knownval(current.receivedAccountCode = bka.code) and
						knownval(current.readyToTransmit = false) and
						knownval(current.hasReplied = false) and
						knownval(current.balanceChecked = true) and
						knownval(current.receivedData = true) and
						knownval(current.receivedAccountPin != bka.pin))
						
						
					do {
						compareInfo_bad();
						conclude((current.correctPin = false), bc:100, fc:100);
						conclude((current.readyToTransmit = true), bc:100, fc:100);
						conclude((bka.balance = current.originalBalance), bc:100, fc:100);						
					}
		}


	

		workframe wf_replyAtm {

					repeat: true;
					
					variables:
 
						forone(Atm) at3;
						foreach(Account) bka;

					
					when(knownval(at3 ownedbyBank current) and
						knownval(bka openedWithBank current) and
						knownval(current.receivedAccountCode = bka.code) and
						known(at3.currentAccountCode) and
						known(at3.currentAccountPin) and
						knownval(current.readyToTransmit = true)) 					
						
					do {
						processReplyAtm();
						replyAtm(at3,bka);
						conclude((current.receivedAccountCode = at3.currentAccountCode), bc:100, fc:100);
						conclude((current.receivedAccountPin = at3.currentAccountPin), bc:100, fc:100);
						conclude((current.readyToTransmit = false), bc:100, fc:100);
						conclude((current.receivedData = false), bc:100, fc:100);
						conclude((current.hasReplied = true), bc:100, fc:100);
						conclude((current.balanceCalculated = false), bc:100, fc:100);
						conclude((current.balanceChecked = false), bc:100, fc:100);
					}
		}


}
