/********************************************************************
 *                   NASA CONFIDENTIAL INFORMATION                  *
 *               (c)1997-2001 Nasa Ames Research Center		    *
 *                        All Rights Reserved                       *
 *                                                                  *
 *  This program contains confidential and proprietary information  *
 *  of NASA Ames Research Center, any reproduction, disclosure,     *
 *  or use in whole or in part is expressly prohibited, except as   *
 *  may be specifically authorized by prior written agreement.      *
 *                                                                  *
 ********************************************************************/

package gov.nasa.arc.brahms.atm;

/*
ATM Model by Alessandro Acquisti acquisti@riacs.edu


COMPLETE VERSION


THE STORY:
This simulation models the simple life of laborious Berkeley students: studying, eating, and getting just enough cash to afford some staple. 
Students spend most of their time studying, but get hungrier as the time goes by. When they are particularly hungry (the level varying depending on the student), they decide to move to one of the two restaurants in town. Students choose the restaurant according to how much money they are carrying. If a student does not have enough money for the cheapest restaurant, she will decide to pass first by the ATM of the bank where she has her account.  

When she arrives at the ATM, the student inserts her bank card and attempt to remember the PIN associated to her account. The ATM gives students 3 attempts to digit the correct PIN, before refusing the card altogether. The ATM communicates with the central bank computer to verify the correctness of the information provided by the student. If the bank computer communicates to the ATM that the PIN is correct and that the student has enough balance in her account, the ATM will dispense the cash and
will print an invoice with the account number and the remaining balance.

Students need to have enough balance in their  accounts to take out
cash: if they attempt to take out more money than they have, the bank computer
will notify the students (through the ATM) of the remaining amount of dollars in the
account. The student will modify her habits accordingly, and take out just exactly the
remaining dollars. 


THE CAST (AGENTS and OBJECTS):
Students: Kim, Alex
Bank computers: Bank of America, Wells Fargo
Restaurants: Blakes, Raleighs
Studying Places: South Hall, Spraul Hall
Clock: the Campanile
ATMs: one ATM for each bank


NOTES:
- attributes and relations scopes are all set to public. private and protected scopes are admitted but not yet implemented in the current version of the language
- the simulation halts at the end of a Berkeley's student hard work day: 20 hours (sleep is not considered!)
- after taking money from the ATMs, student pass by the Hall before getting to the Restaurants
- objects are not given wait activities since detactables/facts do the job (see Tutorial text)


SUGGESTED CHANGES FOR FURTHER MODELING:
- start using private/public/protected scope (see above)
- change ATM and BANK to agents


FURTHER COMPLICATIONS:
- what if each bank has more than one ATM? can you make the agent choose the closest one?
- can  you model the objects (say, Banks and ATMs) as agents? what will change? do you notice changes in the behavior related to detectables and communications?
- what if more students go to the same ATM at the same time? can you make the ATM handle just one order at a time, but the Bank central computer multiple orders coming from several ATMs?
- can you make the agents randomly meet and interact (e.g., say hello each to the other)?
- what if you want the ATM to remember the number of the cards if has refused?


ABOUT THIS FILE:
This file contains the description of the student's behavior. The normal activity of a student is to study. However, as the time goes by
(signalled by the Campanile clock), the students will get hungry and move to the restaurant to eat (or to the ATM machine beforehand, if she does not
have enough cash to pay for the restaurant)


VERSION HISTORY
//VERSION 0.1, February 2001
//VERSION 0.9, April 2001
//VERSION 0.91 April 30, 2001
//VERSION 0.99 August 8, 2001
//VERSION 0.999 December 8, 2001

*/

group Student {

	attributes:
		public boolean male;
		public double howHungry;
		public int perceivedtime;
		public Diner chosenDiner;
		public Bank chosenBank;
		public double preferredCashOut;
		public boolean needCash;
		public boolean checkedDiner;
		public boolean hasTakenCash;
		public boolean hasEaten;
		public boolean readyToLeaveAtm;
		public boolean readyToLeaveRestaurant;
		public boolean pinCommunicated;
		public boolean sessionIsOn;
		public boolean hasCard;
		public boolean calculatedCash;
		public boolean receivedCash;
		public boolean pinRemembered;
		public int correctPin;
		public int believedPin;
		public boolean waitAtmAsksPin;
		public boolean waitAtmAsksAmount;
		public boolean waitAtmRepliesVerification;
		public double amountRequested;
		public boolean amountCommunicated;
		public boolean repeatPin;

		
	relations:
		public Account hasAccount;
		public Cash hasCash;
		public BankCard hasBankCard;

		//ATTRIBUTES AND RELATIONS SCOPES ARE ALL SET TO PUBLIC. PRIVATE AND PROTECTED SCOPES ARE ADMITTED BUT NOT YET IMPLEMENTED IN THE CURRENT VERSION OF THE LANGUAGE

	initial_beliefs:
		(current.checkedDiner = false);
		(Blakes_Diner.location = Telegraph_Av_2134);
		(Raleigh_Diner.location = Telegraph_Av_2405);
		(current.hasEaten = false);
		(current.hasTakenCash = false);
		(current.readyToLeaveRestaurant = false);
		(current.readyToLeaveAtm = false);
		(Boa_Atm.location = Telegraph_Av_113);
		(WF_Atm.location = Bancroft_Av_77);
		(current.pinCommunicated = false);
		(current.sessionIsOn = false);
		(current.hasCard = true);
		(current.calculatedCash = false);
		(current.needCash = false);
		(current.receivedCash = false);
		(current.pinRemembered = false);
		(current.waitAtmAsksPin = false);
		(current.waitAtmAsksAmount = false);
		(current.waitAtmRepliesVerification = false);
		(current.amountCommunicated = false);
		(current.repeatPin = false);

		
	activities:

		move moveToLocation(Building loc) {
			 location: loc;

		}

		primitive_activity study() {
			max_duration: 3000;

		}


		//GOING TO THE RESTAURANT AND EATING IS MODELED AS A COMPOSITE ACTIVITY MADE OF MORE SIMPLER ACTIVITIES
		composite_activity getFood() {

			activities:

				primitive_activity eat() {
						max_duration: 400;
				}

			workframes:

				workframe wf_moveToRestaurant {
					repeat: true;

					variables:
					
						forone(Diner) dn;
						forone(Building) bd;

					
					when(knownval(current.howHungry > 20.00) and
						knownval(current.chosenDiner = dn) and 				        
						not(current.location = dn.location) and  
						knownval(dn.location = bd )) 
				
						
					do {
						moveToLocation(bd);	
						conclude((current.readyToLeaveRestaurant = false), bc:100, fc:0);
					}
				}
		
				workframe wf_eat {

					repeat: true;

					variables:
						forone(Cash) cs; 
						forone(Diner) dn;


					when(knownval(current hasCash cs) and
						 knownval(current.location = dn.location)) 
										
					do {
						
						eat();
						conclude((current.howHungry = current.howHungry - 3.00), bc:100, fc:0);
						conclude((cs.amount = cs.amount - dn.foodcost), bc:100, fc:100); 
						conclude((current.readyToLeaveRestaurant = true), bc:100, fc:0);

					}


				}
			
				
				workframe wf_backToStudy1 {
					repeat: true;

					variables:
						forone(Diner) dn;

					when(knownval(current.readyToLeaveRestaurant = true) and 
						 knownval(current.location = dn.location))

						do {
							moveToLocation(SouthHall);
						} 

				}
		}


		//GOING TO THE ATM AND GETTING CASH IS MODELED AS A COMPOSITE ACTIVITY MADE OF MORE SIMPLER ACTIVITIES
		composite_activity useATM() {

			activities:
				
							  
				primitive_activity insertBankCard() {
					max_duration: 10;
				}
		
				primitive_activity rememberPin() {
					max_duration: 10;
				}	
				//THE ABOVE COULD BE MADE A THOUGHTFRAME ONCE DATAFRAMES ARE USED FOR ATM AND BANKS. SEE NOTE ON DATAFRAMES IN THE ATM FILE

				primitive_activity 	processCommunicatePin() {
					max_duration: 3;
				}

				primitive_activity processSendRequestAmount() {
					max_duration: 3;
				}

				communicate communicatePIN(Atm at3) {
					max_duration: 1;
					with: at3;
					about:
						send(current.pinCommunicated = true),
					//	send(current.believedPin = current.believedPin);
						send(current.believedPin = unknown);
	
					when: end;
				}


				communicate sendRequestAmount(Atm at3) {
					max_duration: 1;
					with: at3;
					about:

					send(current.amountCommunicated = true),
					//send(current.preferredCashOut = current.preferredCashOut);
					send(current.preferredCashOut = unknown);

					when: end;
				}


				primitive_activity waitAtmReply() {
					max_duration: 300;
				}


				primitive_activity switchToWaitForPin() {
					max_duration: 2;
				}

				primitive_activity getCashAndCardAndInvoice() {
					max_duration: 18; 
				}

				primitive_activity doNotGetCashAndInvoice() {
					 max_duration: 14;
				}
			
				primitive_activity finishTransaction() {
					 max_duration: 14;
				}

			workframes:
				
				//THE STUDENT NEEDS FIRST TO MOVE TOWARDS THE ATM OWNED BY THE BANK WHERE SHE HAS HER ACCOUNT....				
				workframe wf_moveToLocationForCash {
					repeat: true;
				
					variables:
						forone(Cash) cs; 
						forone(Atm) at;
						forone(Bank) bk;
						forone(Building) bd;
					
					when(knownval(current hasCash cs) and
						knownval(current.chosenBank = bk) and
						knownval(at ownedbyBank bk) and
						knownval(current.needCash = true) and
						not(current.location = at.location) and
						knownval(at.location = bd ))
						
						
					do {
						moveToLocation(bd);
					}
				}  
					
				
				workframe wf_insertBankCard {
					
					repeat: true; 
					
					variables:
						forone(BankCard) bkc2; 
						forone(Atm) at2;
						forone(Bank) bk2;
						forone(Building) bd2;
					
					when(knownval(current hasBankCard bkc2) and
						knownval(current contains bkc2) and
						knownval(current.chosenBank = bk2) and
						knownval(at2 ownedbyBank bk2) and
						knownval(current.receivedCash = false) and
						knownval(current.needCash = true) and
						knownval(current.location = at2.location) and
						knownval(current.pinRemembered = false)) 
									
					
					do {
						insertBankCard();
						conclude((current contains bkc2 is false), bc:100, fc:100);	
						conclude((at2 contains bkc2), bc:100, fc:100);
						conclude((current.waitAtmAsksPin = true), bc:100, fc:100);

					}
				}

				//THE WAIT ACTIVITIES CAN BE INTERRUPTED BY DETECTABLES. FOR REALISM, ALSO COMMUNICATIONS BETWEEN ATM AND STUDENTS ARE
				//MODELED. (NOTE THAT AGENT VIEWER CRASHES SOMETIMES WHEN OPENING wf WITH DETECTABLES.
				//NOTE THAT SOME ACTIVITIES WILL BE TRIGGERED AS SOON AS THE FACT IS ESTABLISHED. THAT IS WHY THE COMMUNICATION
				//WORKFRAMES IN THIS ATM MODEL ARE USUALLY MODELLED AS SUCH:
				//			1) PROCESS COMMUNICATION (EG, PROCESS PIN)
				//			2) ACTIVATE COMMUNICATION (EG, REPLY PIN)
				//SO THAT THE TIMING OF THE MODEL IS CORRECT
				workframe wf_waitAtmAsksPin {
				
					repeat: true; 

					variables:
						forone(BankCard) bkc4;         
						forone(Cash) cs4; 
						forone(Atm) at4;
						forone(Bank) bk4;
					
					detectables:
						
						detectable atmAsksPin{
							when(whenever)
								detect((at4.pinAsked = true), dc:100)
								then complete;
						}

					
					when(knownval(current.waitAtmAsksPin = true) and
						knownval(current hasBankCard bkc4) and
						knownval(at4 contains bkc4) and
						knownval(current.pinCommunicated = false)) 



					do {
						waitAtmReply();
						conclude((current.waitAtmAsksPin = false), bc:100, fc:100);
											
					}
				
				}


				workframe wf_rememberPin {
					repeat: true;

					variables:
						forone(Account) bka;   
						forone(BankCard) bkc3; 
						forone(Atm) at3;
						forone(Bank) ba3;
						forone(Building) bd3;
				            
					when(knownval(current hasBankCard bkc3) and
						not(current contains bkc3) and
						knownval(current hasAccount bka) and
						knownval(current.chosenBank = ba3) and
						knownval(at3 ownedbyBank ba3) and
						knownval(current.pinCommunicated = false) and
						knownval(current.location = at3.location) and
						knownval(current.pinRemembered = false) and
						knownval(at3.pinAsked = true) and
						knownval(current.pinCommunicated = false))
						
						
					do {
						rememberPin();
						conclude((current.believedPin = bka.pin), bc:100, fc:50);
						conclude((current.believedPin = 9999), bc:50, fc:50);
						conclude((current.pinRemembered = true), bc:100, fc:0);

					}

				}


				workframe wf_communicatePIN {
					
					repeat: true;

					variables:
  
						forone(BankCard) bkc3; 
						forone(Atm) at3;
						forone(Bank) ba3;
						forone(Building) bd3;
					
					when(knownval(current hasBankCard bkc3) and
						not(current contains bkc3) and

						knownval(current.chosenBank = ba3) and
						knownval(at3 ownedbyBank ba3) and
						knownval(current.pinCommunicated = false) and
						knownval(current.location = at3.location) and
						knownval(current.pinRemembered = true) and
						knownval(current.waitAtmAsksPin = false))

						
				//AN EXAMPLE OF WHAT DISCUSSED ABOVE. ATM IS AN OBJECT AND WILL REACT TO FACTS ONLY, NOT BELIEFS
				//BUT THIS ALSO MEANS THAT IT WILL REACT AS SOON AS THE FACT IS ESTABLISHED (AND DETECTED) REGARDLESS
				//OF THE COMMUNICATION WE MODEL. THEREFORE HERE AND IN ALL OTHER WORKFRAMES RELATED TO COMMUNICATION BETWEEN AGENTS OR OBJECTS.
				//WE FOLLOW THIS APPROACH:
				//		1) FIRST, THE COMMUNICATION IS PROCESSED
				//		2) THEN, THE COMMUNICATION IS ACTUALLY PERFORMED AND THE BELIEFS ARE TRANSMITTED
					do {
						processCommunicatePin();
						communicatePIN(at3);
						conclude((current.waitAtmAsksAmount = true), bc:100, fc:100);
						conclude((current.pinCommunicated = true), bc:100, fc:100);
											
					}
				}


				workframe wf_waitAtmAsksAmount {
				
					repeat: true; 

					variables:
						forone(BankCard) bkc4;         
						forone(Cash) cs4; 
						forone(Atm) at4;
						forone(Bank) bk4;
					
					detectables:
						
						detectable atmAsksPin{
							when(whenever)
								detect((at4.amountAsked = true), dc:100)
								then complete;
						}

					
					when(knownval(current.waitAtmAsksAmount = true) and
						knownval(current hasBankCard bkc4) and
						knownval(at4 contains bkc4) and
						knownval(current.pinCommunicated = true) and
						knownval(current.repeatPin = false) and
						knownval(at4.pinAsked = true)) 


					do {
						waitAtmReply();
						conclude((current.waitAtmAsksAmount = false), bc:100, fc:100);
											
					}
				
				}


				//THIS IS USED WHEN THE PIN COMMUNICATED IS WRONG; THE SECOND TIME AROUND THE ATM WILL NOT ASK AGAIN FOR THE AMOUNT
				//THAT THE STUDENT DESIRES TO TAKE OUT, BUT ONLY FOR THE CORRECT PIN. THIS WORKFRAMES SHIFTS THE STUDENT'S ATTENTION FROM
				//BEING ASKED THE AMOUNT TO BEING ASKED THE PIN AGAIN
				workframe wf_switchToWaitForPin {
				
					repeat: true; 

					variables:
						forone(BankCard) bkc4;         
						forone(Cash) cs4; 
						forone(Atm) at4;
						forone(Bank) bk4;

					
					when(knownval(current.waitAtmAsksAmount = true) and
						knownval(current hasBankCard bkc4) and
						knownval(at4 contains bkc4) and
						knownval(current.pinCommunicated = true) and
						knownval(current.repeatPin = true) and
						knownval(at4.pinAsked = true)) 

					do {
						switchToWaitForPin();
						conclude((current.waitAtmAsksAmount = false), bc:100, fc:100);
						conclude((at4.RepliedVerification = false), bc:100, fc:100);
						conclude((current.waitAtmRepliesVerification = true), bc:100, fc:100);											
					}
				
				}


			workframe wf_sendRequestAmount {
					repeat: true;

					variables:
						forone(Account) bka;   
						forone(BankCard) bkc3; 
						forone(Atm) at3;
						forone(Bank) ba3;
						forone(Building) bd3;
				            
					when(knownval(current hasBankCard bkc3) and
						not(current contains bkc3) and
						knownval(current hasAccount bka) and
						knownval(current.chosenBank = ba3) and
						knownval(at3 ownedbyBank ba3) and
						knownval(current.pinCommunicated = true) and
						knownval(current.location = at3.location) and
						knownval(at3.pinAsked = true) and
						knownval(at3.amountAsked = true) and
						knownval(current.amountCommunicated = false))
						
						
					do {
						processSendRequestAmount();
						sendRequestAmount(at3);
						conclude((current.waitAtmRepliesVerification = true), bc:100, fc:100);
						conclude((current.amountCommunicated = true), bc:100, fc:100);

					}

				}

				
			workframe wf_waitAtmRepliesVerification {
				
					repeat: true; 
				
					variables:
						forone(BankCard) bkc4;         
						forone(Cash) cs4; 
						forone(Atm) at4;
						forone(Bank) bk4;


					detectables:
						detectable atmRepliesVerification {
							when(whenever)
								detect((at4.RepliedVerification = true), dc:100)
							//	then abort;
								then complete;
						}

					
					when(knownval(current.waitAtmRepliesVerification = true) and
						knownval(current hasBankCard bkc4) and
						knownval(at4 contains bkc4) and
						knownval(current.amountCommunicated = true)) 

						

					do {
						waitAtmReply();
						conclude((current.waitAtmRepliesVerification = false), bc:100, fc:100);
											
					}
				
				}


				
				workframe wf_doNotGetCashAndInvoice {
				
					repeat: true; 

					variables:
						forone(BankCard) bkc4;         
						forone(Cash) cs4; 
						forone(Atm) at4;
						forone(Bank) bk4;
						forone(Account) bka;
						
					
					when(knownval(current hasBankCard bkc4) and
						knownval(current contains bkc4) and
						knownval(current.location = at4.location) and
						knownval(current.amountCommunicated = true) and
						knownval(at4.cashDispensed = false))

					do {
						doNotGetCashAndInvoice();
						conclude((current.amountCommunicated = false), bc:100, fc:0);
						conclude((current.receivedCash = false), bc:100, fc:0);
						conclude((current.readyToLeaveAtm = true), bc:100, fc:0);
						conclude((at4.cashCanBeDispensed != true), bc:100, fc:0);
						conclude((at4.RepliedVerification = false), bc:100, fc:100);
				
					}
				
				}
				

				workframe wf_getCashAndCardAndInvoice {
				
					repeat: true; 

					variables:
						forone(BankCard) bkc4;         
						forone(Cash) cs4; 
						forone(Atm) at4;
						forone(Bank) bk4;
						forone(Account) bka;
						
					
					when(knownval(current hasBankCard bkc4) and
						knownval(current hasCash cs4) and
						knownval(current contains bkc4) and
						knownval(current.location = at4.location) and
						knownval(current.amountCommunicated = true) and
						knownval(at4.cashDispensed = true))

					do {
						getCashAndCardAndInvoice();
						conclude((cs4.amount = cs4.amount + at4.cashToDispense), bc:100, fc:100); 
						conclude((current.amountCommunicated = false), bc:100, fc:0);
						conclude((current.receivedCash = true), bc:100, fc:0);
						conclude((current.readyToLeaveAtm = true), bc:100, fc:0);
						conclude((at4.cashCanBeDispensed != true), bc:100, fc:0);
						conclude((current.needCash = false), bc:100, fc:100);
						conclude((at4.RepliedVerification = false), bc:100, fc:100);									
					}
				
				}
			
				
			//NOW THE STUDENT WILL LEAVE THE ATM AND THEREAFTER GO STRAIGHT TO THE CHOSEN RESTAURANT	
				workframe wf_finishTransaction {
					repeat: true;
				
					variables:
						forone(Atm) at5;
						forone(Bank) bak;

					when(knownval(current.readyToLeaveAtm = true) and
						knownval(current.location = at5.location) and
						knownval(at5 ownedbyBank bak))

						do {
							
							finishTransaction();
							conclude((current.readyToLeaveAtm = false), bc:100, fc:100);
							conclude((current.receivedCash = false), bc:100, fc:100);
							conclude((current.pinRemembered = false), bc:100, fc:100);
							conclude((current.pinCommunicated = false), bc:100, fc:100);
							conclude((current.amountCommunicated = false), bc:100, fc:100);
							conclude((at5.pinAsked = false), bc:100, fc:0);
							conclude((at5.amountAsked = false), bc:100, fc:0);
							conclude((at5.cashCanBeDispensed != true), bc:100, fc:0);
							conclude((at5.cashDispensed != false), bc:100, fc:0);
							conclude((at5.cashToDispense != 0.0), bc:100, fc:0);
							conclude((current.repeatPin = false), bc:100, fc:0);
							
							
						}

				}
		
			}



	//THESE ARE THE WORKFRAMES RELATED TO THE COMPOSITE ACTIVITIES, OR TO OTHER ACTIVITIES THAT ARE NOT PART THEMSELVES OF 
	//A COMPOSITE ACTIVITY

	workframes:

			workframe wf_study {
					repeat: true;

					
					detectables:	
							detectable  veryHungry {
							when(whenever)
								detect((current.needCash = true), dc:100)
								then abort;
						}

				            
					when(knownval(Campanile_Clock.time < 20) and
						knownval(current.needCash = false) and
						knownval(current.howHungry < 21))	
						
						do { 
							
							study();
							
						}
			}	


			
				workframe wf_useATM {
					
					repeat: true;
        
					
					when(knownval(current.needCash = true) and
						knownval(current.preferredCashOut > 5.00))


					do {
						useATM();
						
					}
				}





			workframe wf_getFood {

				repeat: true;
			
				when(knownval(current.howHungry > 20.00) and
					knownval(current.checkedDiner = true) and
					knownval(current.needCash = false) and
					knownval(current.calculatedCash = true))

				do	{
					getFood();
					conclude((current.calculatedCash = false), bc:100, fc:100);
				}
			}


		thoughtframes:
		
	
			
				//THE FOLLOWING TWO THOUGHTFRAMES ASSESS WHAT RESTAURANT THE STUDENT WILL GO TO, ACCORDING TO THE MONEY SHE IS CARRYING	
				//THESE THOUGHTFRAMES ARE TRIGGERED ONLY TILL THE CAMPANILE REACHES ITS 20-TH HOUR. THEREAFTER, THE SIMULATION HALTS.	
			
			thoughtframe tf_chooseBlakes {
					repeat: true;

					variables:
						forone(Cash) cs;             
					
					when(knownval(current hasCash cs) and
						knownval(cs.amount > 15.00) and
						knownval(current.checkedDiner = false) and
						knownval(Campanile_Clock.time < 20))
							
					do {
						conclude((current.chosenDiner = Blakes_Diner), bc:100);
						conclude((current.checkedDiner = true), bc:100);
					}
				}
		


			thoughtframe tf_chooseRaleigh {
					repeat: true;

					variables:
						forone(Cash) cs;             
					
					when(knownval(current hasCash cs) and
						knownval(cs.amount < 16.00) and
						knownval(current.checkedDiner = false) and
						knownval(Campanile_Clock.time < 20))

							
					do {
						conclude((current.chosenDiner = Raleigh_Diner), bc:100);
						conclude((current.checkedDiner = true), bc:100);
					}
				}
			
			  

			thoughtframe tf_feelHungry {
					repeat: true;

					when(knownval(Campanile_Clock.time > current.perceivedtime))

					do {
						conclude((current.perceivedtime = Campanile_Clock.time), bc: 100);
						conclude((current.howHungry = current.howHungry + 3.00), bc:50);

					}

				}
		

			thoughtframe tf_cash_Needed {
					repeat: true;

					variables:
						forone(Cash) cs; 
						forone(Diner) dn;
				            
				
		
					when(knownval(current.needCash = false) and
						knownval(current.calculatedCash = false) and
						knownval(current.howHungry > 20.00) and
						knownval(current hasCash cs) and
						knownval(current.chosenDiner = dn) and
						knownval(cs.amount < 12.00) and
						knownval(current.preferredCashOut > 0.00))	

					do {
						conclude((current.checkedDiner = false), bc:100);
						conclude((current.needCash = true), bc: 100);
					}

				}

			thoughtframe tf_cash_Not_Needed {
					repeat: true;

					variables:
						forone(Cash) cs; 
						forone(Diner) dn;
				            
				
		
					when(knownval(current.needCash = false) and
						knownval(current.howHungry > 20.00) and
						knownval(current.calculatedCash = false) and
						knownval(current hasCash cs) and
						knownval(current.chosenDiner = dn) and
						knownval(cs.amount > 11.00))	

					do {
						conclude((current.calculatedCash = true), bc:100);
						conclude((current.checkedDiner = false), bc:100);
					}

				}						


}
